/*
 * Copyright (C) 2007 by basyskom GmbH
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef KCALLCORE_H
#define KCALLCORE_H

#include "kcall_export.h"

#include <kabc/stdaddressbook.h>
#include <kabc/addressee.h>

#include <QWidget>
#include <QPixmap>

#include "callinfo.h"

class KStatusBar;
class KXMLGUIClient;
class KAboutData;
class KDialog;
class KActionCollection;
class DecibelDialer;
class DecibelChannelHandler;
class HistoryModel;
class CallModel;
class DTMFInput;

class KCALL_EXPORT KCallCore : public QObject
{
    Q_OBJECT

    public:
        KCallCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
                 const QString &file = QString(), const char *name = 0 );

        ~KCallCore();

        /** Returns parent widget */
        virtual QWidget *widget() const;

        static KAboutData* createAboutData();

        void setStatusBar( KStatusBar *statusBar );
        KStatusBar *statusBar() const;
        void setModified();
        void setModified( bool modified );
        bool modified() const;

        void restoreSettings();
        void saveSettings();
        KConfig *config() const;
        KActionCollection *actionCollection() const;
        virtual KXMLGUIClient *guiClient() const { return m_guiClient; }

        bool queryClose();
        bool handleCommandLine();

        bool newCall(const QString& number);

        // Addressbook:
        KABC::AddressBook * const addressBook() const;
        KABC::Addressee getPersonFromNumber(QString phoneNumber) const;

        // Icons:
        QPixmap getStatusIcon(const CallStatus &);

        // Models:
        HistoryModel * const historyModel() const;
        CallModel * const callModel() const;

    protected slots:
        void showDTMFInput();

    protected:
        void initGUI();
        void initActions();

        QWidget *m_widget;
        KStatusBar *m_statusBar;

        bool m_readWrite;
        bool m_modified;
        bool m_isPart;

        QAction * m_actionSave;

        KABC::AddressBook* const m_addressBook;

        DecibelDialer * const m_decibelDialer;
        DecibelChannelHandler * const m_decibelChannelHandler;
        HistoryModel * const m_historyModel;
        CallModel * const m_callModel;
        DTMFInput * const m_dtmfInput;
        KDialog * m_dtmfInputDialog;

    private:
        KXMLGUIClient *m_guiClient;
        QPixmap m_statusPix[STATUS_MAX];
};

#endif

