/*
 * Copyright (C) 2007 by basyskom GmbH
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <klocale.h>
#include <kstatusbar.h>
#include <kstandardaction.h>
#include <kactioncollection.h>
#include <kshortcutsdialog.h>
#include <kedittoolbar.h>

#include <Decibel/AccountManager>
#include <Decibel/DBusNames>

#include "kcallview.h"
#include "decibeldialer.h"
#include "decibelchannelhandler.h"
#include "kcallcore.h"

#include <QtDBus/QtDBus>
#include "kcalladaptor.h"

#include "kcallmainwindow.h"



KCallMainWindow::KCallMainWindow( const QString& file )
    : KXmlGuiWindow( 0 )
{
  // Set this to be the group leader for all subdialogs - this means
  // modal subdialogs will only affect this dialog, not the other windows
    setAttribute( Qt::WA_GroupLeader );

    setCaption(i18n("KCall"));

    mCore = new KCallCore( this, true, this, file );
    mCore->restoreSettings();

    setCentralWidget( mCore->widget() );

    initActions();

    statusBar()->show();
    statusBar()->insertItem( "", 1 );

    mCore->setStatusBar( statusBar() );

    setStandardToolBarMenuEnabled( true );

    resize( 400, 300 ); // initial size
    setAutoSaveSettings();

    setupGUI();

    new CoreAdaptor( this );
    QDBusConnection::sessionBus().registerObject("/KCall", this, QDBusConnection::ExportAdaptors);
}

KCallMainWindow::~KCallMainWindow()
{
    mCore->saveSettings();
}

void KCallMainWindow::exit()
{
    close();
}

bool KCallMainWindow::newCall(const QString& number)
{
    return mCore->newCall( number );
}

bool KCallMainWindow::handleCommandLine()
{
    return mCore->handleCommandLine();
}

void KCallMainWindow::saveProperties( KConfigGroup& )
{
}

void KCallMainWindow::readProperties( const KConfigGroup& )
{
}

bool KCallMainWindow::queryClose()
{
    return mCore->queryClose();
}


void KCallMainWindow::initActions()
{
    KStandardAction::quit( this, SLOT( close() ), actionCollection() );

    KAction *action;
    action = KStandardAction::keyBindings( this, SLOT( configureKeyBindings() ), actionCollection() );
    action->setWhatsThis( i18n( "You will be presented with a dialog, where you can configure the application wide shortcuts." ) );

    KStandardAction::configureToolbars( this, SLOT( configureToolbars() ), actionCollection() );
}

void KCallMainWindow::configureKeyBindings()
{
    KShortcutsDialog::configure( actionCollection(), KShortcutsEditor::LetterShortcutsAllowed, this );
}

void KCallMainWindow::configureToolbars()
{
    saveMainWindowSettings( KGlobal::config()->group( "MainWindow" ) );

    KEditToolBar edit( factory() );
    connect( &edit, SIGNAL( newToolbarConfig() ),
              this, SLOT( newToolbarConfig() ) );

    edit.exec();
}

void KCallMainWindow::newToolbarConfig()
{
    createGUI( "kcall.rc" );
    applyMainWindowSettings( KGlobal::config()->group( "MainWindow" ) );
}

#include "kcallmainwindow.moc"
