/*
 * Copyright (C) 2006-2007 by basyskom GmbH
 *  @author Mike Hauth <info@basyskom.de>
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef VCARDVIEW_H
#define VCARDVIEW_H

#include <QSortFilterProxyModel>

#include <kabc/addressee.h>
#include "ui_vcardview.h"

class KUrlLabel;
class HistoryModel;

/**
 * This View displays vCard data of an addressee from the kde addressbook.
 * The focus is on data relevant for making phone calls.
 * The phone numbers are clickable.
 */
class VCardView : public QWidget
{
    Q_OBJECT

public:
    explicit VCardView(HistoryModel * const model, QWidget * parent = 0);
    ~VCardView() {}

public slots:
    /** set the addressee to display */
    void setAddressee(KABC::Addressee person);

signals:
    /** The user clicked one of the displays phone numbers of the displayed addressee */
    void phoneNumberClicked( const QString &number);

protected slots:
    void dialNumber(const QString &nr);

protected:
    void setPhoneNumbers(KABC::Addressee person);

    QVBoxLayout *phNrLayout;
    QVBoxLayout *labelLayout;
    QVBoxLayout *nrLayout;
    QList<QLabel*> phoneTypeLabels;
    QList<KUrlLabel*> phoneNumberLabels;

    QSortFilterProxyModel proxyModel;

private:
    Ui::VCardView ui;
};

#endif
