/***************************************************************************
                          notes.cpp  -  description
                             -------------------
    begin                : sob lis 16 2002
    copyright            : (C) 2002-2004 by Micha� Rudolf
    email                : mrudolf@kdewebdev.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "knowit_version.h"
#include "notes.h"
#include <qstringlist.h>
#include <qfile.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <qtextcodec.h>
#include <klocale.h>
#include <k3listview.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kmimetype.h>
#include <krun.h>
#include <kurl.h>
#include <k3listview.h>
#include <KIcon>
#include <KDebug>

void TNoteLink::save(QTextStream& ts) const
{
   if (!link.isEmpty()) {
      ts << "\\Link " << link << "\n";
      if (!description.isEmpty())
         ts << "\\Descr " << description << "\n";
      }
}

QString TNoteLink::text(int fmt) const
{
   if (fmt == LinkOnly || description.isEmpty()) return link;
   else if (fmt == DescriptionOnly) return description;
   else if (fmt == LinkDescription)
      return QString("%1 (%2)").arg(link, description);
   else return QString("%1 (%2)").arg(description, link);
}

QPixmap TNoteLink::icon() const
{
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   return KIcon(KMimeType::iconNameForUrl(KUrl(link))).pixmap( KIcon::Small );
#else
   kWarning() << "Code temporarily removed in TNoteLink::icon";
#endif
   return QPixmap();

}

void TNoteLink::open() const
{
   if (!isLocalReference())
      new KRun(KUrl(link), 0, false, true);
}

void TNoteLink::openWith() const
{
   if (!isLocalReference())
      KRun::displayOpenWithDialog(KUrl::List() << KUrl(link), 0);
}

bool TNoteLink::isLocalReference() const
{
   KUrl url(link);
   return url.protocol() == "knowit";
}





TNotesCollection::TNotesCollection()
{
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   Pixmaps[0] = KIconLoader::global()->loadIcon("ascii", KIcon::Small);
   Pixmaps[1] = KIconLoader::global()->loadIcon("txt", KIcon::Small);
   Pixmaps[2] = KIconLoader::global()->loadIcon("folder", KIcon::Small);
   Pixmaps[3] = KIconLoader::global()->loadIcon("folder_txt", KIcon::Small);
#else
   kWarning() << "Code temporarily removed in TNotesCollection::TNotesCollection";
#endif
   modified = false;
   autoUpdate = true;
}



TNotesCollection::~TNotesCollection()
{}

void TNotesCollection::addNote(Q3ListViewItem* item)
{
   m_notes.insert(item, new TNote(item, this));
   Q3ListViewItem* parent = item->parent();
   if (parent) find(parent)->updateView();
   modified = true;
}

void TNotesCollection::addNote(Q3ListViewItem* item, const QString& s)
{
   m_notes.insert(item, new TNote(item, this, s));
   Q3ListViewItem* parent = item->parent();
   if (parent) find(parent)->updateView();
   modified = true;
}


void TNotesCollection::remove(Q3ListViewItem* item)
{
   m_notes.remove(item);
}

void TNotesCollection::removeNote(Q3ListViewItem* item)
{
   TNote* note = find(item);
   if (!note) qWarning("Internal error while removing note\n");
   else {
      Q3ListViewItem* parent = item->parent();
      for (Q3ListViewItemIterator it(item); it.current() &&
        (it.current() == item || it.current()->depth() > item->depth()); ++it)
         find(it.current())->destroy();
      delete item;
      if (parent) find(parent)->updateView();
      modified = true;
   }
}

bool TNotesCollection::changeNote(Q3ListViewItem* item, const QString& s)
{
   TNote* changed = find(item);
   if (!changed) {
      qWarning("Internal error while changing note\n");
      return false;
   }
   if (changed->change(s))
      return modified = true;
   else
      return false;
}

void TNotesCollection::clearNotes()
{
   qDeleteAll( m_notes.values() );
   m_notes.clear();
}

const QString& TNotesCollection::text(Q3ListViewItem* item)
{
   TNote* note = find(item);
   if (!note)
      qFatal("Internal error while accessing note text\n");
   return note->text;
}

void TNotesCollection::updateNotesView()
{
   Q_FOREACH( TNote* const i, m_notes.values() )
      i->updateView();
}

int TNotesCollection::wordCount(const QString& s)
{
  int words = 0;
  bool inTag = false;
  for (uint i = 0; i < s.length(); i++)
  {
    if (inTag && s[i] == '>')
      inTag = false;
    else if (!inTag && s[i] == '<')
      inTag = true;
   else if (!inTag && s[i].isLetter() && (!i || !s[i-1].isLetter()))
      words++;
  }
  return words;
}

int TNotesCollection::charCount(const QString& s)
{
  int chars = 0;
  bool inTag = false;
  for (uint i = 0; i < s.length(); i++)
  {
    if (inTag && s[i] == '>')
      inTag = false;
    else if (!inTag && s[i] == '<')
      inTag = true;
    else if (!inTag && s[i] != '\n')
      chars++;
    else if (!inTag && s[i] == '\n' && i && s[i-1] != '>' && s[i+1] != '<')
      chars++;
  }
  return chars;
}

int TNotesCollection::parsCount(const QString& text)
{
  return text.count("<p ") + text.count("<p>");
}


TNote::TNote(Q3ListViewItem* i, TNotesCollection* coll) : text()
{
   collection = coll;
   item = i;
   updateView();
}

TNote::TNote(Q3ListViewItem* i, TNotesCollection* coll, const QString& s) :
   text(s)
{
   collection = coll;
   item = i;
   updateView();
}

bool TNote::isEmpty()
{
   return !text.length();
}

bool TNote::contains(const QString& sought, bool case_sensitive)
{
   return text.contains(sought, case_sensitive ? Qt::CaseSensitive : Qt::CaseInsensitive );
}


TNote::~TNote()
{
}

void TNote::destroy()
{
   collection->remove(item);
   delete this;
}

bool TNote::change(const QString& s)
{
   if (text == s)
      return false;
   text = s;
   updateView();
   return true;
}

TNote::State TNote::state()
{
   if (text.trimmed().isEmpty())
      return (item->childCount()) ? EmptyParent : Empty;
   else
      return (item->childCount()) ? TextParent : Text;
}

void TNote::updateView()
{
   if (collection->autoUpdate)
      item->setPixmap(0, collection->Pixmaps[state()]);
}

bool TNote::saveHTML(const KUrl& fname, const QString& origname, const QString& style,
   int flags)
{
  QFile file(fname.path());
  if (!file.open( QIODevice::WriteOnly))
     return false;
  QTextStream ts(&file);
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   if (flags & UseUTF8) ts.setEncoding(QTextStream::UnicodeUTF8);
   else ts.setEncoding(QTextStream::Locale);
#else
   kWarning() << "Code temporarily removed in TNote::saveHTML";
#endif

  ts << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
     "<html>\n<head>\n   <title>" << i18n("KnowIt document") <<
     QString("</title>\n   <meta http-equiv=\"Content-Type\" "
     "content=\"text/html; charset=%1\">\n").arg( QString::fromLatin1(ts.codec()->name()));
  if (!style.isEmpty())
     ts << "   <style type=\"text/css\">\n   " << style << "\n   </style>\n";
  ts << "</head>\n\n<body>\n";

  QVector<int> Depths;
  Depths.append(0);
  if (flags & TOC) {
     ts << QString("<h1>%1</h1>\n").arg(i18n("Table of contents"));
     saveHTMLTocEntry(ts, Depths, flags);
     ts << "\n\n";
     }

  Depths.clear();
  Depths.append(0);
  saveHTMLBuf(ts, Depths, flags);

  ts << "<br><br><br><small>" << i18n("Exported from %1 by KnowIt %2, %3.", origname,
     VERSION,
     QDateTime::currentDateTime().toString()) << "</small>\n\n";
  ts << "</body>\n</html>\n\n";
  return true;
}

bool TNote::saveHTMLBuf(QTextStream& ts, QVector<int>& depths,
  int flags)
{
   int hlevel = depths.count();
   depths.last() = depths.last()+1;
   if (hlevel < 1) hlevel = 1;
   else if (hlevel > 3) hlevel = 3;
   QString id, number;
   for (uint i=0; i<depths.count(); i++)
         id += QString("%1.").arg(depths[i]);
   if (flags & Enumerate)
      number = id + " ";

   ts << QString("<h%1 id=\"S%2\">%3%4</h%5>").arg(QString::number(hlevel),
                                                   id,
                                                   number,
                                                   item->text(0),
                                                   QString::number(hlevel));
   QString htmltext = text;
   int begin = htmltext.indexOf("<body");
   if (begin >= 0) begin = htmltext.indexOf(">", begin);
   int end = htmltext.indexOf("</body>");
   if (begin < 0) begin = 0;
   if (end <= begin) end = htmltext.length();
   ts << htmltext.mid(begin+1, end-begin-1);

   /* save links */
   for (QList<TNoteLink>::Iterator LinkList = links.begin();
      LinkList != links.end(); ++LinkList)
      ts << QString("<a href=\"%1\">%2</a><br>\n").arg((*LinkList).link,
         (*LinkList).text(TNoteLink::DescriptionOnly));

   /* save rule */
   if (flags & AddRule) ts << "<hr size=\"1\" noshade>\n\n";
   else ts << "\n\n";

   /* save children */
   if ((SaveSubnotes | SaveAll) & flags && item->childCount()) {
      depths.append(0);
      collection->find(item->firstChild())->saveHTMLBuf(ts, depths, flags | SaveAll);
      }
   if (SaveAll & flags && item->nextSibling())
      collection->find(item->nextSibling())->saveHTMLBuf(ts, depths, flags);

   if (!item->nextSibling()) depths.pop_back();
   return true;
}

bool TNote::saveHTMLTocEntry(QTextStream& ts, QVector<int>& depths, int flags)
{
   QString space(depths.count(), ' ');
   depths.last() = depths.last()+1;
   if (depths.last() == 1) {
      ts << space;
      if (depths.count() != 1) ts << "<dd>";
      ts << "<dl>\n";
      }

   QString id, number;
   for (uint i=0; i<depths.count(); i++)
         id += QString("%1.").arg(depths[i]);
   if (flags & Enumerate) number = id + " ";

   ts << space;
   ts << QString("<dt><a href=\"#S%1\">%2%3</a></dt>\n").arg(id, number).
      arg(item->text(0));
   if ((SaveSubnotes | SaveAll) & flags && item->childCount()) {
      depths.append(0);
      collection->find(item->firstChild())->saveHTMLTocEntry(ts, depths, flags | SaveAll);
      }
   if (SaveAll & flags && item->nextSibling())
      collection->find(item->nextSibling())->saveHTMLTocEntry(ts, depths, flags);
   if (!item->nextSibling()) {
      depths.pop_back();
      ts << space;
      ts << "</dl>";
      if (depths.count()) ts << "</dd>";
      ts << "\n";
      }
   return true;
}

void TNote::addLink(const QString& s)
{
   links.append(TNoteLink(s));
   collection->modified = true;
}

void TNote::addLink(const TNoteLink& l)
{
   links.append(l);
   collection->modified = true;
}

void TNote::removeLink(int i)
{
   links.removeAll(links[i]);
   collection->modified = true;
}

void TNote::modifyLink(int i, const QString& s)
{
   links[i].link = s;
   collection->modified = true;
}

void TNote::modifyLink(int i, TNoteLink& l)
{
   if (l != links[i])
      collection->modified = true;
   links[i] = l;
}

void TNote::modifyLinkDescription(int i, const QString& s)
{
   links[i].description = s;
   collection->modified = true;
}

int TNote::linkCount() const
{
   return links.count();
}

const TNoteLink& TNote::link(int i) const
{
   return links[i];
}


void TNote::save(QTextStream& ts, bool current)
{
   const QString header = "\\NewEntry %1 %2\n";
   const QString activeheader = "\\CurrentEntry %1 %2\n";

   if (current)
      ts << activeheader.arg(QString::number(item->depth()), item->text(0));
   else
      ts << header.arg(QString::number(item->depth()), item->text(0));
   Q_FOREACH( const TNoteLink& LinkList, links )
        LinkList.save(ts);
   ts << text;
   if (!text.isEmpty())
      ts << '\n';
   ts << '\n';
}

void TNote::open(QTextStream& ts)
{
   QString s;
   text = "";
   links.clear();

   while (true) {
      s = ts.readLine();
      if (s.isEmpty())
         break;
      if (s.left(6) == "\\Link ")
         addLink(s.mid(6));
      else if (s.left(7) == "\\Descr " && links.count())
         modifyLinkDescription(links.count()-1, s.mid(7));
      else {
         if (text.length())
            text += '\n';
         text += s;
      }
   }
}
