/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_ITEMSTATUSMODIFYJOB_H
#define KRSS_ITEMSTATUSMODIFYJOB_H

#include "krss_export.h"
#include "item.h"

#include <KJob>

namespace KRss {

class Feed;
class ItemStatusModifyJobPrivate;

class KRSS_EXPORT ItemStatusModifyJob : public KJob
{
    Q_OBJECT

public:

    enum Error {
        FeedLoadFailed = KJob::UserDefinedError,
        ItemStatusModificationFailed
    };

public:

    explicit ItemStatusModifyJob( QObject *parent = 0 );
    ~ItemStatusModifyJob();

    QList<Item> items() const;
    void setItems( const QList<Item>& items );

    void clearFlags( const QList<KRss::StatusFlags> &flags );
    void setFlags( const QList<KRss::StatusFlags> &flags );

    void start();
    QString errorString() const;

private:

    void processItems();
    void checkDone();

private Q_SLOTS:

    void doStart();
    void slotItemModificationDone( KJob* );

private:

    ItemStatusModifyJobPrivate * const d;
};

} // namespace KRss

#endif // KRSS_ITEMSTATUSMODIFYJOB_H
