/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "netfeedcreatejob.h"
#include "krssinterface.h"
#include <KDebug>

#include <QtCore/QTimer>

using namespace KRss;

namespace KRss {

class NetFeedCreateJobPrivate
{
public:

    explicit NetFeedCreateJobPrivate( const QString &xmlUrl, const QString &subscriptionLabel,
                                      const QString &resourceIdentifier )
        : m_xmlUrl( xmlUrl ), m_subscriptionLabel( subscriptionLabel ),
          m_resourceIdentifier( resourceIdentifier )
    {
    }

    const QString m_xmlUrl;
    const QString m_subscriptionLabel;
    const QString m_resourceIdentifier;
};

} // namespace KRss

NetFeedCreateJob::NetFeedCreateJob( const QString &xmlUrl, const QString &subscriptionLabel,
                                    const QString &resourceIdentifier, QObject *parent ) :
    KJob( parent ), d( new NetFeedCreateJobPrivate( xmlUrl, subscriptionLabel, resourceIdentifier ) )
{
}

NetFeedCreateJob::~NetFeedCreateJob()
{
    delete d;
}

void NetFeedCreateJob::start()
{
    QTimer::singleShot( 0, this, SLOT( doStart() ) );
}

QString NetFeedCreateJob::errorString() const
{
    switch ( error() ) {
        case DBusCallQueueingFailed:
        case FeedCreationFailed:
            return errorText();
        default:
            return KJob::errorString();
    }

    return QString();
}

void NetFeedCreateJob::doStart()
{
    org::kde::krss *interface = new org::kde::krss( "org.freedesktop.Akonadi.Agent." + d->m_resourceIdentifier, "/KRss",
                                                    QDBusConnection::sessionBus(), this );

    // don't block, set callbacks instead
    QList<QVariant> argumentList;
    argumentList << qVariantFromValue( d->m_xmlUrl ) << qVariantFromValue( d->m_subscriptionLabel );
    if ( !interface->callWithCallback( QLatin1String( "addFeed" ), argumentList,
                            this, SLOT( slotCallFinished( qlonglong ) ), SLOT( slotCallFailed() ) ) ) {
        setError( DBusCallQueueingFailed );
        setErrorText( "Failed to place a D-Bus call");
        emitResult();
    }
}

void NetFeedCreateJob::slotCallFinished( qlonglong id )
{
    kDebug() << "D-Bus call returned:" << id;

    emitResult();
}

void NetFeedCreateJob::slotCallFailed()
{
    setError( FeedCreationFailed );
    setErrorText( "Failed to add a new feed to the resource" );
    emitResult();
}

#include "netfeedcreatejob.moc"
