/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_PERSISTENTFEED_H
#define KRSS_PERSISTENTFEED_H

#include "krss_export.h"
#include "feed.h"

class QPixmap;
class KJob;

namespace KRss {

class Item;
class Resource;
class FeedCollection;
class FeedListPrivate;
class PersistentFeedItemListJobImpl;
class PersistentFeedPrivate;
class RetrieveFeedListJobPrivate;

class KRSS_EXPORT PersistentFeed : public Feed
{
    Q_OBJECT

    friend class ::KRss::PersistentFeedPrivate;
    friend class ::KRss::PersistentFeedItemListJobImpl;

public:

    ~PersistentFeed();

    Feed::Id id() const;

    QString title() const;
    void setTitle( const QString &title );
    QString description() const;
    void setDescription( const QString &description );
    QString type() const;
    void setType( const QString &type );
    KIcon icon() const;
    void setIcon( const KIcon &icon );
    QList<TagId> tags() const;
    void setTags( const QList<TagId> &tags );
    void addTag( const TagId &tag );
    void removeTag( const TagId &tag );
    QPixmap image() const;
    void setImage( const QPixmap &pixmap );

    QStringList subscriptionLabels() const;
    void setSubscriptionLabels( const QStringList &subscriptionLabels );
    void addSubscriptionLabel( const QString &subscriptionLabel );
    void removeSubscriptionLabel( const QString &subscriptionLabel );

    int total() const;
    int unread() const;

    int fetchInterval() const;
    void setFetchInterval( int minutes );
    int maxArticleNumber() const;
    void setMaxArticleNumber( int number );
    int maxArticleAge() const;
    void setMaxArticleAge( int age );

    virtual ItemListJob* createListJob();

public Q_SLOTS:

    void fetch() const;
    void abortFetch() const;

protected:

    explicit PersistentFeed( const FeedCollection &feedCollection, const Resource *resource, QObject *parent = 0 );
    FeedCollection& feedCollection() const;

private Q_SLOTS:

    void slotCollectionLoadDone( KJob *job );
    void slotStatisticsFetchDone( KJob* job );

    void slotFeedChanged( KRss::Feed::Id id );
    void slotFeedRemoved( KRss::Feed::Id id );

    void slotFetchStarted( KRss::Feed::Id id );
    void slotFetchPercent( KRss::Feed::Id id, uint percentage );
    void slotFetchFinished( KRss::Feed::Id id );
    void slotFetchFailed( KRss::Feed::Id id, const QString &errorMessage );
    void slotFetchAborted( KRss::Feed::Id id );

    void slotItemAdded( const Akonadi::Item &item, KRss::Feed::Id id );
    void slotItemChanged( const Akonadi::Item &item );
    void slotItemRemoved( const Akonadi::Item &item );

    void slotStatisticsChanged( KRss::Feed::Id id, const Akonadi::CollectionStatistics& stats );

private:

    Q_DISABLE_COPY(PersistentFeed)

    friend class ::KRss::RetrieveFeedListJobPrivate;
    friend class ::KRss::FeedListPrivate;
    friend class PersistentFeedFetchJob;
    friend class PersistentFeedModifyJobPrivate;
    friend class PersistentFeedDeleteJobPrivate;
    PersistentFeedPrivate * const d;
};

} // namespace KRss

#endif // KRSS_PERSISTENTFEED_H
