/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "treenode.h"

#include <KDebug>

using namespace KRss;

TreeNode::TreeNode( TreeNode::Tier tier, Feed *feed, TreeNode *parent )
    : m_tier( tier ), m_feed ( feed ), m_parent( parent )
{
}

TreeNode::~TreeNode()
{
    removeChildren();
}

void TreeNode::appendChild( TreeNode *child )
{
    m_children << child;
}

void TreeNode::removeChildAt( int row )
{
    TreeNode *child = childAt( row );
    delete child;
    m_children.removeAt( row );
}

void TreeNode::removeChildren()
{
    qDeleteAll( m_children );
    m_children.clear();
}

int TreeNode::count() const
{
    return m_children.count();
}

TreeNode* TreeNode::childAt( int row ) const
{
    return m_children.value( row );
}

int TreeNode::childRow( TreeNode *child ) const
{
    return m_children.indexOf( child );
}

QList<TreeNode*> TreeNode::children() const
{
    return m_children;
}

TreeNode* TreeNode::parent() const
{
    return m_parent;
}

Feed* TreeNode::feed() const
{
    return m_feed;
}

TreeNode::Tier TreeNode::tier() const
{
    return m_tier;
}

int TreeNode::row() const
{
    // Since 'this' is of 'const TreeNode *const' type I can't just pass it
    // to QList::indexOf() which accepts 'TreeNode *'
    // Shame on me but I can't figure out a correct way to pass the pointer
    // to this object other than casting away constness.
    // Though this should be safe.
    return ( m_parent ? m_parent->childRow( const_cast<TreeNode*>( this ) ) : 0 );
}

void TreeNode::dump( const QString &indent ) const
{
    switch( m_tier ) {
        case RootNode:
            kDebug() << QString( "%1%2" ).arg( indent ).arg( "Root node" );
            break;

        case TagNode:
            kDebug() << QString( "%1%2" ).arg( indent ).arg( m_feed->title() );
            break;

        case FeedNode:
            kDebug() << QString( "%1Feed, id %2" ).arg( indent ).arg( m_feed->title() );
            break;
    }

    Q_FOREACH( const TreeNode *child, m_children )
        child->dump( indent + "    " );
}
