/*
    Copyright (C) 2008, 2009    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rssresourcebase.h"
#include "krssadaptor.h"
#include "libkrss/resourcemanager.h"
#include "rssbackend.h"
#include "rssbackendjobs.h"

#include <KDE/Akonadi/CollectionFetchJob>
#include <KDE/Akonadi/CollectionModifyJob>
#include <KDE/Akonadi/CollectionDeleteJob>
#include <KDE/Akonadi/ChangeRecorder>

#include <KDE/KDebug>
#include <KDE/KLocale>

#include <QtDBus/QDBusConnection>

using namespace KRss;

// member functions
RssResourceBase::RssResourceBase( const QString &id )
        :ResourceBase( id )
{
    KRss::ResourceManager::registerAttributes();

    changeRecorder()->fetchCollection( true );

    new KrssAdaptor( this );
    if ( !QDBusConnection::sessionBus().registerObject( QLatin1String( "/KRss" ),
         this, QDBusConnection::ExportAdaptors ) ) {
        kWarning() << "Couldn't register a D-Bus service org.kde.krss";
        kWarning() << QDBusConnection::sessionBus().lastError().message();
    }
}

void RssResourceBase::collectionAdded( const Akonadi::Collection &collection, const Akonadi::Collection &parent )
{
    changeCommitted( collection );
}

void RssResourceBase::collectionChanged( const Akonadi::Collection &collection )
{
    changeCommitted( collection );
}

void RssResourceBase::collectionRemoved( const Akonadi::Collection &collection )
{
    changeCommitted( collection );
}

void RssResourceBase::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
{
    changeCommitted( item );
}

void RssResourceBase::itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers )
{
    changeCommitted( item );
}

void RssResourceBase::itemRemoved( const Akonadi::Item &item )
{
    changeCommitted( item );
}

KRss::Feed::Id RssResourceBase::addFeed( const QString &xmlUrl, const QString &subscriptionLabel )
{
    return 0;
}

void RssResourceBase::fetchFeed( const KRss::Feed::Id& id )
{
    return;
}

void RssResourceBase::abortFetch( const KRss::Feed::Id& id )
{
    return;
}

bool RssResourceBase::removeFeed( const KRss::Feed::Id& id )
{
    return false;
}

bool RssResourceBase::subscribe( const KRss::Feed::Id& id, const QString &subscriptionLabel )
{
    return false;
}

bool RssResourceBase::unsubscribe( const KRss::Feed::Id& id, const QString &subscriptionLabel )
{
    return false;
}

QStringList RssResourceBase::subscriptions( const KRss::Feed::Id& id )
{
    return QStringList();
}

bool RssResourceBase::addTag( const KRss::Feed::Id& id, const QString &tag )
{
    return false;
}

bool RssResourceBase::removeTag( const KRss::Feed::Id& id, const QString &tag )
{
    return false;
}

QStringList RssResourceBase::tags( const KRss::Feed::Id& id )
{
    return QStringList();
}

bool RssResourceBase::importOpml( const QString &path, const QString &defaultTag )
{
    return false;
}

void RssResourceBase::exportOpml( const QString &path )
{
    return;
}

bool RssResourceBase::retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
    return false;
}

void RssResourceBase::configure( WId windowId )
{
    Q_UNUSED( windowId )

    // don't do anything
    return;
}

void RssResourceBase::retrieveCollections()
{
    if ( rssBackend() ) {
        FeedsRetrieveJob * const job = rssBackend()->feedsRetrieveJob();
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotCollectionsRetrieved( KJob* ) ) );
        job->start();
    }
}

void RssResourceBase::retrieveItems( const Akonadi::Collection &col )
{
    return;
}

void RssResourceBase::slotCollectionsRetrieved( KJob *job )
{
    if ( job->error() ) {
        kWarning() << job->errorString();
        collectionsRetrievedIncremental( Akonadi::Collection::List(), Akonadi::Collection::List() );
        return;
    }

    FeedsRetrieveJob *fjob = dynamic_cast<FeedsRetrieveJob*>( job );
    Q_ASSERT( fjob );

    collectionsRetrieved( fjob->feeds() );
}

#include "rssresourcebase.moc"
